@@ -7,7 +7,7 @@ use std::path::{Path, PathBuf};
 
 use directories::{BaseDirs, ProjectDirs};
 use once_cell::sync::Lazy;
-use serde::Deserialize;
+use serde::{Deserialize, Serialize};
 
 use tor_error::{ErrorKind, HasKind};
 
@@ -27,18 +27,32 @@ use tor_error::{ErrorKind, HasKind};
 /// so should use appropriate system-specific overrides under the
 /// hood. (Some of those overrides are based on environment variables.)
 /// For more information, see that crate's documentation.
-#[derive(Clone, Debug, Deserialize, Eq, PartialEq)]
+///
+/// Alternatively, a `CfgPath` can contain literal `PathBuf`, which will not be expaneded.
+#[derive(Clone, Debug, Serialize, Deserialize, Eq, PartialEq)]
 #[serde(transparent)]
 pub struct CfgPath(PathInner);
 
 /// Inner implementation of CfgPath
-#[derive(Clone, Debug, Deserialize, Eq, PartialEq)]
+///
+/// `PathInner` exists to avoid making the variants part of the public Rust API
+#[derive(Clone, Debug, Serialize, Deserialize, Eq, PartialEq)]
 #[serde(untagged)]
 enum PathInner {
+    /// A path that should be used literally, with no expansion.
+    Literal(LiteralPath),
     /// A path that should be expanded from a string using ShellExpand.
     Shell(String),
-    /// A path that should be used literally, with no expansion.
-    Literal(PathBuf),
+}
+
+#[derive(Clone, Debug, Serialize, Deserialize, Eq, PartialEq)]
+/// Inner implementation of PathInner:Literal
+///
+/// `LiteralPath` exists to arrange that `PathInner::Literal`'s (de)serialization
+/// does not overlap with `PathInner::Shell`'s.
+struct LiteralPath {
+    ///
+    literal: PathBuf,
 }
 
 /// An error that has occurred while expanding a path.
@@ -91,17 +105,43 @@ impl CfgPath {
         CfgPath(PathInner::Shell(s))
     }
 
+    /// Construct a new `CfgPath` designating a literal not-to-be-expanded `PathBuf`
+    pub fn new_literal<P: Into<PathBuf>>(path: P) -> Self {
+        CfgPath(PathInner::Literal(LiteralPath {
+            literal: path.into(),
+        }))
+    }
+
     /// Return the path on disk designated by this `CfgPath`.
     pub fn path(&self) -> Result<PathBuf, CfgPathError> {
         match &self.0 {
             PathInner::Shell(s) => expand(s),
-            PathInner::Literal(p) => Ok(p.clone()),
+            PathInner::Literal(LiteralPath { literal }) => Ok(literal.clone()),
         }
     }
 
-    /// Construct a new `CfgPath` from a system path.
-    pub fn from_path<P: AsRef<Path>>(path: P) -> Self {
-        CfgPath(PathInner::Literal(path.as_ref().to_owned()))
+    /// If the `CfgPath` is a string that should be expaneded, return the (unexpanded) string,
+    ///
+    /// Before use, this string would have be to expanded.  So if you want a path to actually use,
+    /// call `path` instead.
+    ///
+    /// Returns `None` if the `CfgPath` is a literal `PathBuf` not intended for expansion.
+    pub fn as_unexpanded_str(&self) -> Option<&str> {
+        match &self.0 {
+            PathInner::Shell(s) => Some(s),
+            PathInner::Literal(_) => None,
+        }
+    }
+
+    /// If the `CfgPath` designates a literal not-to-be-expanded `Path`, return a reference to it
+    ///
+    /// Returns `None` if the `CfgPath` is a string which should be expanded, which is the
+    /// usual case.
+    pub fn as_literal_path(&self) -> Option<&Path> {
+        match &self.0 {
+            PathInner::Shell(_) => None,
+            PathInner::Literal(LiteralPath { literal }) => Some(literal),
+        }
     }
 }
 
@@ -153,7 +193,7 @@ fn get_env(var: &str) -> Result<Option<&'static str>, CfgPathError> {
 impl std::fmt::Display for CfgPath {
     fn fmt(&self, fmt: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match &self.0 {
-            PathInner::Literal(p) => write!(fmt, "{:?} [exactly]", p),
+            PathInner::Literal(LiteralPath { literal }) => write!(fmt, "{:?} [exactly]", literal),
             PathInner::Shell(s) => s.fmt(fmt),
         }
     }
@@ -256,7 +296,7 @@ mod test {
 
     #[test]
     fn literal() {
-        let p = CfgPath::from_path(PathBuf::from("${ARTI_CACHE}/literally"));
+        let p = CfgPath::new_literal(PathBuf::from("${ARTI_CACHE}/literally"));
         // This doesn't get expanded, since we're using a literal path.
         assert_eq!(
             p.path().unwrap().to_str().unwrap(),
@@ -265,3 +305,127 @@ mod test {
         assert_eq!(p.to_string(), "\"${ARTI_CACHE}/literally\" [exactly]");
     }
 }
+
+#[cfg(test)]
+mod test_serde {
+    use super::*;
+    use std::ffi::OsString;
+    use std::fmt::Debug;
+
+    #[derive(Serialize, Deserialize, Eq, PartialEq, Debug)]
+    struct TestConfigFile {
+        p: CfgPath,
+    }
+
+    fn deser_json(json: &str) -> CfgPath {
+        dbg!(json);
+        let TestConfigFile { p } = serde_json::from_str(json).expect("deser json failed");
+        p
+    }
+    fn deser_toml(toml: &str) -> CfgPath {
+        dbg!(toml);
+        let TestConfigFile { p } = toml::from_str(toml).expect("deser toml failed");
+        p
+    }
+    fn deser_toml_cfg(toml: &str) -> CfgPath {
+        dbg!(toml);
+        let cfg = config::File::from_str(toml, config::FileFormat::Toml);
+        let cfg = config::Config::builder()
+            .add_source(cfg)
+            .build()
+            .expect("parse toml failed");
+        dbg!(&cfg);
+        let TestConfigFile { p } = cfg.try_deserialize().expect("deser cfg failed");
+        p
+    }
+
+    #[test]
+    fn test_parse() {
+        fn desers(toml: &str, json: &str) -> Vec<CfgPath> {
+            vec![deser_toml(toml), deser_toml_cfg(toml), deser_json(json)]
+        }
+
+        for cp in desers(r#"p = "string""#, r#"{ "p": "string" }"#) {
+            assert_eq!(cp.as_unexpanded_str(), Some("string"));
+            assert_eq!(cp.as_literal_path(), None);
+        }
+
+        for cp in desers(
+            r#"p = { literal = "lit" }"#,
+            r#"{ "p": {"literal": "lit"} }"#,
+        ) {
+            assert_eq!(cp.as_unexpanded_str(), None);
+            assert_eq!(cp.as_literal_path(), Some(&*PathBuf::from("lit")));
+        }
+    }
+
+    fn non_string_path() -> PathBuf {
+        #[cfg(target_family = "unix")]
+        {
+            use std::os::unix::ffi::OsStringExt;
+            return PathBuf::from(OsString::from_vec(vec![0x80_u8]));
+        }
+
+        #[cfg(target_family = "windows")]
+        {
+            use std::os::windows::ffi::OsStringExt;
+            return PathBuf::from(OsString::from_wide(&[0xD800_u16]));
+        }
+
+        #[allow(unreachable_code)]
+        // Cannot test non-Stringy Paths on this platform
+        PathBuf::default()
+    }
+
+    fn test_roundtrip_cases<SER, S, DESER, E, F>(ser: SER, deser: DESER)
+    where
+        SER: Fn(&TestConfigFile) -> Result<S, E>,
+        DESER: Fn(&S) -> Result<TestConfigFile, F>,
+        S: Debug,
+        E: Debug,
+        F: Debug,
+    {
+        let case = |easy, p| {
+            let input = TestConfigFile { p };
+            let s = match ser(&input) {
+                Ok(s) => s,
+                Err(e) if easy => panic!("ser failed {:?} e={:?}", &input, &e),
+                Err(_) => return,
+            };
+            dbg!(&input, &s);
+            let output = deser(&s).expect("deser failed");
+            assert_eq!(&input, &output, "s={:?}", &s);
+        };
+
+        case(true, CfgPath::new("string".into()));
+        case(true, CfgPath::new_literal(PathBuf::from("nice path")));
+        case(true, CfgPath::new_literal(PathBuf::from("path with ✓")));
+
+        // Non-UTF-8 paths are really hard to serialize.  We allow the serializsaton
+        // to fail, and if it does, we skip the rest of the round trip test.
+        // But, if they did serialise, we want to make sure that we can deserialize.
+        // Hence this test case.
+        case(false, CfgPath::new_literal(non_string_path()));
+    }
+
+    #[test]
+    fn roundtrip_json() {
+        test_roundtrip_cases(
+            |input| serde_json::to_string(&input),
+            |json| serde_json::from_str(json),
+        );
+    }
+
+    #[test]
+    fn roundtrip_toml() {
+        test_roundtrip_cases(|input| toml::to_string(&input), |toml| toml::from_str(toml));
+    }
+
+    #[test]
+    fn roundtrip_mpack() {
+        test_roundtrip_cases(
+            |input| rmp_serde::to_vec(&input),
+            |mpack| rmp_serde::from_slice(mpack),
+        );
+    }
+}
